// KioskModeInfo -- a class that represents the information
// about a query to a web API, together with a stored JSONObject
// representing the result of the most recent query.

class KioskModeInfo
{
  String type;
  String q1;
  String q2;

  JSONObject data;
  int last_updated;
  int seconds_per_update;

  KioskModeInfo( String type, String q1, String q2 )
  {
    this.type = type;
    this.q1 = q1;
    this.q2 = q2;

    data = null;
    // Update once per hour.
    seconds_per_update = 3600;
    
    if( type.equals( "GRT" ) ) {
      // Unless it's GRT data.  Then update every 
      // 30 seconds, since buses are always on the
      // move.
      seconds_per_update = 30;
    }
  }

  // A shortcut for passing in just two arguments.
  KioskModeInfo( String type, String q1 )
  {
    this( type, q1, "" );
  }

  // A method that sends you back this Kiosk mode's
  // JSONObject, so that you can display it.  This is
  // basically the only thing you need to do with a
  // KioskInfoMode after you create it.
  
  JSONObject getData()
  {
    boolean do_update = false;
    if ( data == null ) {
      // If we don't currently have a query result,
      // then we definitely have to update.
      do_update = true;
    } else {
      // Even if we *do* have a query result, we might
      // still want to update if enough time has elapsed
      // since the last update.
      int tm = millis();
      if ( (tm - last_updated) > (seconds_per_update * 1000) ) {
        do_update = true;
      }
    }

    // If we decided that it's time to update, actually
    // retrieve the data.
    if ( do_update ) {
      doLoadData();
      // ...and reset the clock.
      last_updated = millis();
    }

    return data;
  }

  // Never call this directly.  Use getData().  This
  // method does the actual querying and stores the
  // resulting JSONObject.
  
  void doLoadData()
  {
    if ( type == "UW" ) {
      data = requestUW( q1 );
    } else if ( type == "GRT" ) {
      data = requestGRT( q1, q2 );
    } else if ( type == "RSS" ) {
      data = requestRSS( q1 );
    } else if ( type == "TEST" ) {
      // Let's have a simple TEST mode in which you
      // get back a pretend JSONObject with some test
      // data in it.
      data = new JSONObject();
      data.setBoolean( "success", true );
      data.setInt( "an_integer", int( random( 10 ) ) );
      data.setFloat( "a_float", random( 1 ) );
      JSONArray arr = new JSONArray();
      arr.setInt( 0, int(random(10,100)) );
      arr.setInt( 1, int(random(10,100)) );
      arr.setInt( 2, int(random(10,100)) );
      data.setJSONArray( "an_array", arr );
    }
  }

  // Get JSON data from the UW API.
  JSONObject requestUW( String query )
  {  
    String baseurl = "https://api.uwaterloo.ca/v2";
    // This is our shared key.  Please treat it gently.
    String apikey = "6b5e445e121719d5b816623ed57545cd";

    if ( query.contains( "?" ) ) {
      return loadJSONObject( baseurl + query + "&key=" + apikey );
    } else {
      return loadJSONObject( baseurl + query + "?key=" + apikey );
    }
  }

  // Get JSON data from Grand River Transit.  This is the
  // API that powers their realtimemap.grt.ca site.
  JSONObject requestGRT( String stop_number, String route_number )
  {
    String baseurl = "http://realtimemap.grt.ca/Stop/GetStopInfo";
    return loadJSONObject( baseurl + "?stopId=" + stop_number
      + "&routeId=" + route_number );
  }

  // Convert an RSS feed into JSON using a service provided by
  // Google.
  JSONObject requestRSS( String rss_url )
  {
    String baseurl = "https://ajax.googleapis.com/ajax/services/feed/load?v=2.0&q=";
    return loadJSONObject( baseurl + rss_url );
  }
}

